/* Light mapping using multipass rendering */
/* An OpenGL demo by A.P.Gardner */

#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>

/* variables */

GLuint texName[2];		/* store of texture references */

/* Function to load a texture from a bmp file */

void makeTextureImage(char filename[], int size, GLuint tex_name)
{
	long data_start = 54;			/* start of data section in files wall.bmp & light.bmp */
									/* NB header size of bmp files is not fixed, refer to M$ documentation */
	unsigned char ch, cho[4];		/* temp file input buffers */
	FILE *fileptr;					/* pointer to bmp file */
	GLubyte image256[256][256][3];	/* storage for 256x256 texture */
	GLubyte image32[32][32][3];		/* storage for 32x32 texture */
	int n, m, i, j;					/* counters */

	/* open file */

	if ((fileptr = fopen(filename, "rb")) == NULL)
		return;

	/* step over the header */

	n = 0;
	while (!feof(fileptr) && n < data_start)
	{
		ch = fgetc(fileptr);
		n++;
	}

	/* load the texture data */

	for (i=0; i<size; i++)
	{
		for (j=0; j<size; j++)
		{
			m=0;
			while(!feof(fileptr) && m < 3)
			{
				cho[m] = fgetc(fileptr);
				m++;
			}
			if (size == 32)
			{
				image32[i][j][0] = cho[2];	/* bmp using BGR values, hence the swap */
				image32[i][j][1] = cho[1];
				image32[i][j][2] = cho[0];
			}
			else if (size == 256)
			{
				image256[i][j][0] = cho[2];
				image256[i][j][1] = cho[1];
				image256[i][j][2] = cho[0];
			}
		}
	}

	/* close the file */

	fclose(fileptr);

	/* create texture object */
	
	glBindTexture(GL_TEXTURE_2D, texName[tex_name]);

	/* set up wrap & filter params */

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	/* load the image */

	if (size == 32)
	{
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, size, size,
					 0, GL_RGB, GL_UNSIGNED_BYTE, image32);
	}
	else
	{
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, size, size,
					 0, GL_RGB, GL_UNSIGNED_BYTE, image256);
	}
}

/* Function to initialise params */

void init(void)
{
	/* set background colour */

	glClearColor(0.0, 0.0, 0.0, 0.0);

	/* initialise viewing parameters */

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0.0, 1.0, 0.0, 1.0, -1.0, 1.0);

	/* generate texture names */

	texName[0] = 0;
	texName[1] = 0;
	glGenTextures(2, texName);

	/* load textures */

	makeTextureImage("light.bmp", 32, 0);
	makeTextureImage("wall.bmp", 256, 1);

	/* enable texturing & set texturing function */

	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
}

/* Function to clean up */

void cleanup(void)
{
	/* delete texture objects */

	glDeleteTextures(2, texName);
}

/* Function to draw display */

void display(void)
{
	/* clear all pixels in colour buffer */

	glClear(GL_COLOR_BUFFER_BIT);

	/* bind light map texture */

	glBindTexture(GL_TEXTURE_2D, texName[0]);

	/* draw a quad */

	glBegin(GL_QUADS);
		glTexCoord2f(0.03125, 0.03125);
		glVertex3f(0.2, 0.2, 0.0);
		glTexCoord2f(0.65625, 0.03125);
		glVertex3f(0.8, 0.2, 0.0);
		glTexCoord2f(0.65625, 0.65625);
		glVertex3f(0.8, 0.8, 0.0);
		glTexCoord2f(0.03125, 0.65625);
		glVertex3f(0.2, 0.8, 0.0);
	glEnd();

	/* enable blending */

	glEnable(GL_BLEND);

	/* set blend function to multiply SRC & DST colours */

	glBlendFunc(GL_ZERO, GL_SRC_COLOR);

	/* bind wall texture */

	glBindTexture(GL_TEXTURE_2D, texName[1]);

	/* draw a quad */

	glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);
		glVertex3f(0.2, 0.2, 0.0);
		glTexCoord2f(1.0, 0.0);
		glVertex3f(0.8, 0.2, 0.0);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(0.8, 0.8, 0.0);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(0.2, 0.8, 0.0);
	glEnd();

	/* disable blending */

	glDisable(GL_BLEND);

	/* process all buffered OpenGL commands */

	glFlush();
}

/* Main function */

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
	glutInitWindowSize(500, 500);
	glutInitWindowPosition(100, 100);
	glutCreateWindow("OpenGL Demo - Light Mapping (Multipass)");
	init();
	glutDisplayFunc(display);
	glutMainLoop();
	cleanup();

	return 0;
}