/*
	multiview.c
	Nate Robins, 1997
	
	Program that shows how to use multiple viewports in a single
	context (using scissoring).
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#define CPWDLL_EXTERN
#include <cpw.h>

GLuint torus_list = 0;
GLfloat spin_x = 0.0;
GLfloat spin_y = 0.0;
CpwFontFace font;
CpwWindowInfo info;

void
text(pCpw cpw, char* string)
{
    cpwDrawFont( cpw, font, string, 1 );
}

void
lists(pCpw cpw)
{
    GLfloat gold_Ka[4] = { 0.24725f, 0.1995f, 0.0745f, 1.0f };
    GLfloat gold_Kd[4] = { 0.75164f, 0.60648f, 0.22648f, 1.0f };
    GLfloat gold_Ks[4] = { 0.628281f, 0.555802f, 0.366065f, 1.0f };
    GLfloat gold_Ke    = 41.2f;
    GLfloat silver_Ka[4]  = { 0.05f, 0.05f, 0.05f, 1.0f };
    GLfloat silver_Kd[4]  = { 0.4f, 0.4f, 0.4f, 1.0f };
    GLfloat silver_Ks[4]  = { 0.7f, 0.7f, 0.7f, 1.0f };
    GLfloat silver_Ke     = 12.0f;
    
    torus_list = glGenLists(1);
    glNewList(torus_list, GL_COMPILE);
    glMaterialfv(GL_FRONT, GL_AMBIENT,  gold_Ka);
    glMaterialfv(GL_FRONT, GL_DIFFUSE,  gold_Kd);
    glMaterialfv(GL_FRONT, GL_SPECULAR, gold_Ks);
    glMaterialf(GL_FRONT, GL_SHININESS, gold_Ke);
    glMaterialfv(GL_BACK, GL_AMBIENT,  silver_Ka);
    glMaterialfv(GL_BACK, GL_DIFFUSE,  silver_Kd);
    glMaterialfv(GL_BACK, GL_SPECULAR, silver_Ks);
    glMaterialf(GL_BACK, GL_SHININESS, silver_Ke);
    cpwDrawPrimitive( cpw,CPW_PRIM_3D_WIRETORUS );
    glEndList();
}

void
init(void)
{
    GLfloat light_pos[4] = { 1.0, 1.0, 1.0, 0.0 };

    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
    glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);
}

void
reshape(pCpw cpw, uint_32 id, int width, int height)
{
    glClearColor(0.0, 0.0, 0.0, 0.0);
    info.width = width;
    info.height = height;
}

void
projection(int width, int height, int perspective)
{
    float ratio = (float)width/height;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if (perspective)
      gluPerspective(60, ratio, 1, 256);
    else 
      glOrtho(-ratio, ratio, -ratio, ratio, 1, 256);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(0.0, 0.0, 2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
}

#define bottom_left                   \
    glViewport(0, 0, width, height);  \
    glScissor(0, 0, width, height)

#define bottom_right                     \
    glViewport(width, 0, width, height); \
    glScissor(width, 0, width, height)

#define top_left                          \
    glViewport(0, height, width, height); \
    glScissor(0, height, width, height)

#define top_right                             \
    glViewport(width, height, width, height); \
    glScissor(width, height, width, height)

#define front                         \
    projection(width, height, 0);     \
    glRotatef(spin_y, 1.0, 0.0, 0.0); \
    glRotatef(spin_x, 0.0, 1.0, 0.0)

#define back                          \
    projection(width, height, 0);     \
    glRotatef(180.0, 0.0, 1.0, 0.0);  \
    glRotatef(spin_y, 1.0, 0.0, 0.0); \
    glRotatef(spin_x, 0.0, 1.0, 0.0)

#define right                            \
    projection(width, height, 0);        \
    glRotatef(90.0, 0.0, 1.0, 0.0);      \
    glRotatef(spin_y, 1.0, 0.0, 0.0);    \
    glRotatef(spin_x, 0.0, 1.0, 0.0)

#define left                             \
    projection(width, height, 0);        \
    glRotatef(-90.0, 0.0, 1.0, 0.0);      \
    glRotatef(spin_y, 1.0, 0.0, 0.0);    \
    glRotatef(spin_x, 0.0, 1.0, 0.0)

#define top                               \
    projection(width, height, 0);         \
    glRotatef(90.0, 1.0, 0.0, 0.0);       \
    glRotatef(spin_y, 1.0, 0.0, 0.0);     \
    glRotatef(spin_x, 0.0, 1.0, 0.0)

#define bottom                            \
    projection(width, height, 0);         \
    glRotatef(-90.0, 1.0, 0.0, 0.0);      \
    glRotatef(spin_y, 1.0, 0.0, 0.0);     \
    glRotatef(spin_x, 0.0, 1.0, 0.0)

#define perspective                           \
    projection(width, height, 1);             \
    glRotatef(30.0, 0.0, 1.0, 0.0);           \
    glRotatef(20.0, 1.0, 0.0, 0.0);           \
    glRotatef(spin_y, 1.0, 0.0, 0.0);         \
    glRotatef(spin_x, 0.0, 1.0, 0.0)

void
display(pCpw cpw, uint_32 id )
{
    int width = info.width;
    int height = info.height;

    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, width, 0, height);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glDisable(GL_LIGHTING);
    glColor3ub(255, 255, 255);
    glBegin(GL_LINES);
    glVertex2i(width/2,0);
    glVertex2i(width/2,height);
    glVertex2i(0,height/2);
    glVertex2i(width,height/2);
    glEnd();

    glRasterPos2i(5, 5);
    text(cpw,"Front");
    glRasterPos2i(width/2+5, 5);
    text(cpw,"Right");
    glRasterPos2i(5, height/2+5);
    text(cpw,"Top");
    glRasterPos2i(width/2+5, height/2+5);
    text(cpw,"Perspective");

    glEnable(GL_LIGHTING);

    width = (width+1)/2;
    height = (height+1)/2;

    glEnable(GL_SCISSOR_TEST);

    bottom_left;
    front;
    glCallList(torus_list);
    
    bottom_right;
    right;
    glCallList(torus_list);

    top_left;
    top;
    glCallList(torus_list);

    top_right;
    perspective;
    glCallList(torus_list);

    glDisable(GL_SCISSOR_TEST);

    cpwSwapBuffers(cpw);
}

void
skeyboard(pCpw cpw, uint_32 winid, unsigned char key, uint_32 state, int x, int y)
{
    switch (key) {
    case CPW_KEY_ESCAPE:
	exit(0);
	break;
    }

    cpwPostRedisplay(cpw);
}

void
keyboard(pCpw cpw, uint_32 winid, unsigned char key, uint_32 state, int x, int y)
{
    switch( key ) {

        case '1':
          cpwClearEvents( cpw ); 
        break;
        case '2':
          cpwClearEvent( cpw, CPW_MOUSEMOTION_EVENT ); 
        break;
        case '3':
          cpwClearWindowEvents( cpw, winid );
        break;
        case '4':
        break;

    }
}

int old_x, old_y;

void
mouse(pCpw cpw,uint_32 winid, int button, int state, int x, int y)
{
    old_x = x;
    old_y = y;

    cpwPostRedisplay(cpw);
}

void
motion(pCpw cpw,int x, int y)
{
    spin_x = x - old_x;
    spin_y = y - old_y;

    cpwPostRedisplay(cpw);
}

int
main(int argc, char** argv)
{
    pCpw cpw = null;

    cpwInitContext(&cpw);

    cpwInitWindowPosition(cpw,50, 50);
    cpwInitWindowSize(cpw,512, 512);

    cpwCreateWindow(cpw,"Multiple Viewports");
    cpwSystemKeyboardCallback(cpw,skeyboard);
    cpwKeyboardCallback(cpw,keyboard);
    cpwReshapeCallback(cpw,reshape);
    cpwDisplayCallback(cpw,display);
    cpwMouseMoveCallback(cpw,motion);
    cpwMouseClickCallback(cpw,mouse);

    cpwSetPrimitiveOpt(cpw,CPW_PRIMOPT_INNERRADIUS,0.3);
    cpwSetPrimitiveOpt(cpw,CPW_PRIMOPT_OUTERRADIUS,0.7);
    cpwSetPrimitiveOpt(cpw,CPW_PRIMOPT_SLICES,20);
    cpwSetPrimitiveOpt(cpw,CPW_PRIMOPT_STACKS,20);

    cpwFontMode( cpw, CPW_FONTOPT_ALIGNLEFT, 0 );
    cpwFontMode( cpw, CPW_FONTOPT_PIXELMAP_GLFORMAT, GL_LUMINANCE );
    cpwFontMode( cpw, CPW_FONTOPT_SIZE, 20 );
    //font = cpwLoadFont( cpw, "TIMES.TTF", CPW_FONTLOC_HOST, "", "" );
    font = cpwLoadFont( cpw, "ARIAL.TTF", CPW_FONTLOC_HOST, "", "" );

    init();
    lists(cpw);

    cpwMainLoop(cpw);

    cpwFreeContext(&cpw);
    return 0;
}
