/* 
    null.c
    Nate Robins, 1997

    An example of using null bitmaps to place the rasterpos at
    positions that may be clipped.

 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CPWDLL_EXTERN
#include <cpw.h>


unsigned char image[256*256*3];
unsigned char bitmap[1] = { 0 };
int raster_x = 32;
int raster_y = 32;
int old_raster_x;
int old_raster_y;

void
reshape(pCpw cpw,int width, int height)
{
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, width, 0, height, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void
display(pCpw cpw)
{
    glClear(GL_COLOR_BUFFER_BIT);

    glRasterPos2i(0, 0);
    glBitmap(0, 0, 0, 0, raster_x, raster_y, NULL);
    glDrawPixels(256, 256, GL_RGB, GL_UNSIGNED_BYTE, (GLvoid*)image);

    cpwSwapBuffers(cpw);
}

void
mouse(pCpw cpw, uint_32 id, int button, int state, int x, int y)
{
    y = cpwGet(cpw,CPW_WINDOW_HEIGHT) - y;

    old_raster_x = x - raster_x;
    old_raster_y = y - raster_y;
}

void
motion(pCpw cpw,uint_32 id,int x, int y)
{
    y = cpwGet(cpw,CPW_WINDOW_HEIGHT) - y;

    raster_x = x - old_raster_x;
    raster_y = y - old_raster_y;

    cpwPostRedisplay(cpw);
}

void
skeyboard(pCpw cpw,unsigned char key, uint_32 state, int x, int y)
{
    if (key == CPW_KEY_ESCAPE)
	exit(0);
}

int
main(int argc, char** argv)
{
    int i, j;
    pCpw cpw;

    cpw = null;

    cpwInitContext( &cpw );

    //glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA);
    cpwInitWindowPosition(cpw,50, 50);
    cpwInitWindowSize(cpw,320, 320);
    //glutInit(&argc, argv);

    cpwCreateWindow(cpw,"null");
    cpwSystemKeyboardCallback(cpw,skeyboard);
    cpwDisplayCallback(cpw,display);
    cpwReshapeCallback(cpw,reshape);
    cpwMouseMoveCallback(cpw,motion);
    cpwMouseClickCallback(cpw,mouse);

    /* create a pretty color ramp */
    for (j = 0; j < 256; j++) {
	for (i = 0; i < 256; i++) {
	    image[(256 * j + i) * 3 + 0] = 255-i*j/255;
	    image[(256 * j + i) * 3 + 1] = i;
	    image[(256 * j + i) * 3 + 2] = j;
	}
    }

    cpwMainLoop(cpw);

    cpwFreeContext( &cpw );
    return 0;
}
