/* 
    qix.c
    Nate Robins, 1997

    An example of a 'qix'-like line demo, but without the traditional
    erase lines, and with anti-aliased lines.

 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CPWDLL_EXTERN
#include <cpw.h>

typedef struct _point {
    float x, y;
    struct _point* next;
} point;


point* first;
point* second;
int    num_points = 73;
float  first_vx = 0.223487f;
float  first_vy = 0.532978f;
float  second_vx = 0.63257f;
float  second_vy = 0.325897f;
float  vstep = 27.232743f;
CpwWindowInfo info;

#define COLORS 48
GLubyte colors[COLORS][3] = {
    255, 0, 0,
    255, 32, 0,
    255, 64, 0,
    255, 96, 0,
    255, 128, 0,
    255, 160, 0,
    255, 192, 0,
    255, 224, 0,
    255, 255, 0,
    224, 255, 0,
    192, 255, 0,
    160, 255, 0,
    128, 255, 0,
    96, 255, 0,
    64, 255, 0,
    32, 255, 0,
    0, 255, 0,
    0, 255, 32,
    0, 255, 64,
    0, 255, 96,
    0, 255, 128,
    0, 255, 160,
    0, 255, 192,
    0, 255, 224,
    0, 255, 255,
    0, 224, 255,
    0, 196, 255,
    0, 160, 255,
    0, 128, 255,
    0, 96, 255,
    0, 64, 255,
    0, 32, 255,
    0, 0, 255,
    32, 0, 255,
    64, 0, 255,
    96, 0, 255,
    128, 0, 255,
    160, 0, 255,
    196, 0, 255,
    224, 0, 255,
    255, 0, 255,
    255, 0, 224,
    255, 0, 196,
    255, 0, 160,
    255, 0, 128,
    255, 0, 96,
    255, 0, 64,
    255, 0, 32
};


void
reshape(pCpw cpw, uint_32 id, uint_32 width, uint_32 height)
{
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, width, 0, height, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    //glEnable(GL_BLEND);
    //glEnable(GL_LINE_SMOOTH);
    //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    info.width = width;
    info.height = height;
}

void
display(pCpw cpw, uint_32 id)
{
    int i;
    static int color = 0;
    
    glClear(GL_COLOR_BUFFER_BIT);

    first->next->x = first->x + first_vx;
    first->next->y = first->y + first_vy;
    first = first->next;
    if (first->x < 0) {
	first->x = 0;
	first_vx = -first_vx;
    } else if (first->x > info.width ) {
	first->x = info.width;
	first_vx = -first_vx;
    }
    if (first->y < 0) {
	first->y = 0;
	first_vy = -first_vy;
    } else if (first->y > info.height) {
	first->y = info.height;
	first_vy = -first_vy;
    }
    
    second->next->x = second->x + second_vx;
    second->next->y = second->y + second_vy;
    second = second->next;
    if (second->x < 0) {
	second->x = 0;
	second_vx = -second_vx;
    } else if (second->x > info.width) {
	second->x = info.width;
	second_vx = -second_vx;
    }
    if (second->y < 0) {
	second->y = 0;
	second_vy = -second_vy;
    } else if (second->y > info.height) {
	second->y = info.height;
	second_vy = -second_vy;
    }
    
    glBegin(GL_LINES);
    for (i = 0; i < num_points; i++) {
	glVertex2i(first->x, first->y);
	glVertex2i(second->x, second->y);
	glColor3ubv(colors[(color+i) % COLORS]);
	first = first->next;
	second = second->next;
    }
    glEnd();

    color++;
    if (color >= COLORS)
	color = 0;

    cpwSwapWindowBuffers(cpw, id);

}

void
idle(pCpw cpw)
{
    cpwPostWindowRedisplay(cpw, 1);
}

void
bail(int code)
{
    int i;
    point* nuke;

    for (i = 0; i < num_points; i++) {
	nuke = first;
	first = first->next;
	free(nuke);
    }
    for (i = 0; i < num_points; i++) {
	nuke = second;
	second = second->next;
	free(nuke);
    }
    exit(code);
}

void
keyboard(pCpw cpw, uint_32 winid, uint_32 key, uint_32 state, uint_32 x, uint_32 y)
{
    static int old_x = 50;
    static int old_y = 50;
    static int old_width = 320;
    static int old_height = 320;

    if (key == 27) {
        bail(0);
    } else if (key == 'w') {
        cpwPositionWindow(cpw, winid, old_x, old_y);
        cpwReshapeWindow(cpw, winid, old_width, old_height);
    } else if (key == 'f') {
	if (cpwGet(cpw,CPW_WINDOW_WIDTH) != cpwGet(cpw,CPW_SCREEN_WIDTH)) {
	    old_x = cpwGet(cpw,CPW_WINDOW_X);
	    old_y = cpwGet(cpw,CPW_WINDOW_Y);
	    old_width = cpwGet(cpw,CPW_WINDOW_WIDTH);
	    old_height = cpwGet(cpw,CPW_WINDOW_HEIGHT);
	    cpwFullscreenWindow(cpw,1);
	}
    }
}

int
main(int argc, char** argv)
{
    int i;
    point* newborn;
    pCpw cpw = null;

    cpwInitContext(&cpw);
    cpwInitWindowPosition(cpw,50, 50);
    cpwInitWindowSize(cpw,100, 100);

    cpwCreateWindow(cpw,"Qix");
    cpwDisplayCallback(cpw,display);
    cpwReshapeCallback(cpw,reshape);
    cpwKeyboardCallback(cpw,keyboard);

    if (argc > 1) {
	if (strcmp(argv[1], "-h") == 0) {
	    fprintf(stderr, "%s [lines] [step]\n", argv[0]);
	    exit(0);
	}
	sscanf(argv[1], "%d", &num_points);
	if (argc > 2)
	    sscanf(argv[2], "%d", &vstep);
    }      

    first_vx *= vstep;
    first_vy *= vstep;
    second_vx *= vstep;
    second_vy *= vstep;

    for (i = 0; i < num_points; i++) {
	newborn = (point*)malloc(sizeof(point));
	newborn->next = first;
	newborn->x = -1.0;
	newborn->y = -1.0;
	first = newborn;
    }

    newborn = first;
    while(newborn->next)
	newborn = newborn->next;
    newborn->next = first;

    first->x = rand() % cpwGet(cpw,CPW_WINDOW_WIDTH);
    first->y = rand() % cpwGet(cpw,CPW_WINDOW_HEIGHT);

    for (i = 0; i < num_points; i++) {
	newborn = (point*)malloc(sizeof(point));
	newborn->next = second;
	newborn->x = -1.0;
	newborn->y = -1.0;
	second = newborn;
    }

    newborn = second;
    while(newborn->next)
	newborn = newborn->next;
    newborn->next = second;

    second->x = rand() % cpwGet(cpw,CPW_WINDOW_WIDTH);
    second->y = rand() % cpwGet(cpw,CPW_WINDOW_HEIGHT);

    cpwIdleCallback(cpw,idle);
    cpwMainLoop(cpw);

    cpwFreeContext(&cpw);

    return 0;
}
